#include <pnm_env.h>
#include <assert.h>

template <class T> class Pyramid {

	unsigned  levels;
	T *  data; 
	
protected:	

  void downsample(unsigned lvl){

  T &img1 = data[lvl];
  T &img2 = data[lvl+1];

  T tmp(img1.GetWidth()/2, img1.GetHeight());
  
  img2 = T(img1.GetWidth()/2, img1.GetHeight()/2);
  
  unsigned x, y;

  // width down sample
  for (x=0; x<tmp.GetWidth(); x++){
    for (y=0; y<tmp.GetHeight(); y++){
      tmp.Pix(x,y) = (img1.cPix(x*2-1,y) + img1.cPix(x*2+1,y)) * 0.25F + 
	               img1.cPix(x*2,y) * 0.5F;
    }
  }

  // height down sample
  for (x=0; x<img2.GetWidth(); x++){
    for (y=0; y<img2.GetHeight(); y++){
      img2.Pix(x,y) = (tmp.cPix(x,y*2-1) + tmp.cPix(x,y*2+1)) * 0.25F + 
	               tmp.cPix(x,y*2) * 0.5F;
    }
  }
}

public:

	T & operator[](unsigned i){
		assert(i>=0 && i<levels);
		return data[i];
	}

	const T & operator[](unsigned i) const {
		assert(i>=0 && i<levels);
		return data[i];
	}

	Pyramid(const T &img, int lvl): levels(lvl)
	{
		data = new T[levels];

		data[0] = img;
		
		for (unsigned i = 0; i < levels -1 ; i++)
			downsample(i);
	}

	~Pyramid()
	{
		for (unsigned i=0; i<levels;i++)
			data[i] = T();				// null assignment frees data
		delete [] data;
	}
};



int main(int argc, char *argv[])
{

	fRgbImg img;

	img = pnm(argv[1]);
	Pyramid<fRgbImg> pyr(img,3);

	pnm((bRgbImg) pyr[0]).Write("pyr0.ppm");
	pnm((bRgbImg) pyr[1]).Write("pyr1.ppm");
	pnm((bRgbImg) pyr[2]).Write("pyr2.ppm");
	
	return 0;
}

